@@ -60,8 +60,6 @@ oidc_cache_mutex_t *oidc_cache_mutex_create(apr_pool_t *pool) {
 	oidc_cache_mutex_t *ctx = apr_pcalloc(pool, sizeof(oidc_cache_mutex_t));
 	ctx->mutex = NULL;
 	ctx->mutex_filename = NULL;
-	ctx->shm = NULL;
-	ctx->sema = NULL;
 	ctx->is_parent = TRUE;
 	return ctx;
 }
@@ -71,9 +69,10 @@ oidc_cache_mutex_t *oidc_cache_mutex_create(apr_pool_t *pool) {
 /*
  * convert a apr status code to a string
  */
-char *oidc_cache_status2str(apr_status_t statcode) {
+char *oidc_cache_status2str(apr_pool_t *p, apr_status_t statcode) {
 	char buf[OIDC_CACHE_ERROR_STR_MAX];
-	return apr_strerror(statcode, buf, OIDC_CACHE_ERROR_STR_MAX);
+	apr_strerror(statcode, buf, OIDC_CACHE_ERROR_STR_MAX);
+	return apr_pstrdup(p, buf);
 }
 
 apr_byte_t oidc_cache_mutex_post_config(server_rec *s, oidc_cache_mutex_t *m,
@@ -82,21 +81,28 @@ apr_byte_t oidc_cache_mutex_post_config(server_rec *s, oidc_cache_mutex_t *m,
 	apr_status_t rv = APR_SUCCESS;
 	const char *dir;
 
-	// oidc_sdebug(s, "enter: %d (m=%pp,s=%pp, p=%d)", (m && m->sema) ? *m->sema : -1, m->mutex ? m->mutex : 0, s, m->is_parent);
-
 	/* construct the mutex filename */
 	apr_temp_dir_get(&dir, s->process->pool);
 	m->mutex_filename = apr_psprintf(s->process->pool,
 			"%s/mod_auth_openidc_%s_mutex.%ld.%pp", dir, type,
 			(long int) getpid(), s);
 
+	/* set the lock type */
+	apr_lockmech_e mech =
+#ifdef OIDC_LOCK
+			OIDC_LOCK
+#elif APR_HAS_POSIXSEM_SERIALIZE
+			APR_LOCK_POSIXSEM
+#else
+			APR_LOCK_DEFAULT
+#endif
+			;
+
 	/* create the mutex lock */
-	rv = apr_global_mutex_create(&m->mutex, (const char *) m->mutex_filename,
-			APR_LOCK_DEFAULT, s->process->pool);
+	rv =
+			apr_global_mutex_create(&m->mutex, (const char*) m->mutex_filename, mech, s->process->pool);
 	if (rv != APR_SUCCESS) {
-		oidc_serror(s,
-				"apr_global_mutex_create failed to create mutex on file %s: %s (%d)",
-				m->mutex_filename, oidc_cache_status2str(rv), rv);
+		oidc_serror(s, "apr_global_mutex_create failed to create mutex (%d) on file %s: %s (%d)", mech, m->mutex_filename, oidc_cache_status2str(s->process->pool, rv), rv);
 		return FALSE;
 	}
 
@@ -110,23 +116,12 @@ apr_byte_t oidc_cache_mutex_post_config(server_rec *s, oidc_cache_mutex_t *m,
 	if (rv != APR_SUCCESS) {
 		oidc_serror(s,
 				"unixd_set_global_mutex_perms failed; could not set permissions: %s (%d)",
-				oidc_cache_status2str(rv), rv);
+				oidc_cache_status2str(s->process->pool, rv), rv);
 		return FALSE;
 	}
 #endif
 
-	apr_global_mutex_lock(m->mutex);
-
-	rv = apr_shm_create(&m->shm, sizeof(int), NULL, s->process->pool);
-	if (rv != APR_SUCCESS) {
-		oidc_serror(s, "apr_shm_create failed to create shared memory segment");
-		return FALSE;
-	}
-
-	m->sema = apr_shm_baseaddr_get(m->shm);
-	*m->sema = 1;
-
-	apr_global_mutex_unlock(m->mutex);
+	oidc_slog(s, APLOG_TRACE1, "create: %pp (m=%pp,s=%pp, p=%d)", m, m->mutex ? m->mutex : 0, s, m->is_parent);
 
 	return TRUE;
 }
@@ -137,7 +132,7 @@ apr_byte_t oidc_cache_mutex_post_config(server_rec *s, oidc_cache_mutex_t *m,
 apr_status_t oidc_cache_mutex_child_init(apr_pool_t *p, server_rec *s,
 		oidc_cache_mutex_t *m) {
 
-	// oidc_sdebug(s, "enter: %d (m=%pp,s=%pp, p=%d)", (m && m->sema) ? *m->sema : -1, m->mutex ? m->mutex : 0, s, m->is_parent);
+	oidc_slog(s, APLOG_TRACE1, "init: %pp (m=%pp,s=%pp, p=%d)", m, m->mutex ? m->mutex : 0, s, m->is_parent);
 
 	if (m->is_parent == FALSE)
 		return APR_SUCCESS;
@@ -149,16 +144,10 @@ apr_status_t oidc_cache_mutex_child_init(apr_pool_t *p, server_rec *s,
 	if (rv != APR_SUCCESS) {
 		oidc_serror(s,
 				"apr_global_mutex_child_init failed to reopen mutex on file %s: %s (%d)",
-				m->mutex_filename, oidc_cache_status2str(rv), rv);
-	} else {
-		apr_global_mutex_lock(m->mutex);
-		m->sema = apr_shm_baseaddr_get(m->shm);
-		(*m->sema)++;
-		apr_global_mutex_unlock(m->mutex);
+				m->mutex_filename, oidc_cache_status2str(p, rv), rv);
 	}
 
 	m->is_parent = FALSE;
-	//oidc_sdebug(s, "semaphore: %d (m=%pp,s=%pp)", *m->sema, m, s);
 
 	return rv;
 }
@@ -172,7 +161,7 @@ apr_byte_t oidc_cache_mutex_lock(server_rec *s, oidc_cache_mutex_t *m) {
 
 	if (rv != APR_SUCCESS)
 		oidc_serror(s, "apr_global_mutex_lock() failed: %s (%d)",
-				oidc_cache_status2str(rv), rv);
+				oidc_cache_status2str(s->process->pool, rv), rv);
 
 	return TRUE;
 }
@@ -186,7 +175,7 @@ apr_byte_t oidc_cache_mutex_unlock(server_rec *s, oidc_cache_mutex_t *m) {
 
 	if (rv != APR_SUCCESS)
 		oidc_serror(s, "apr_global_mutex_unlock() failed: %s (%d)",
-				oidc_cache_status2str(rv), rv);
+				oidc_cache_status2str(s->process->pool, rv), rv);
 
 	return TRUE;
 }
@@ -198,35 +187,12 @@ apr_byte_t oidc_cache_mutex_destroy(server_rec *s, oidc_cache_mutex_t *m) {
 
 	apr_status_t rv = APR_SUCCESS;
 
-	// oidc_sdebug(s, "enter: %d (m=%pp,s=%pp, p=%d)", (m && m->sema) ? *m->sema : -1, m->mutex ? m->mutex : 0, s, m->is_parent);
+	oidc_slog(s, APLOG_TRACE1, "init: %pp (m=%pp,s=%pp, p=%d)", m, m->mutex ? m->mutex : 0, s, m->is_parent);
 
-	if (m->mutex != NULL) {
-
-		apr_global_mutex_lock(m->mutex);
-		(*m->sema)--;
-		//oidc_sdebug(s, "semaphore: %d (m=%pp,s=%pp)", *m->sema, m->mutex, s);
-
-		// oidc_sdebug(s, "processing: %d (m=%pp,s=%pp, p=%d)", (m && m->sema) ? *m->sema : -1, m->mutex ? m->mutex : 0, s, m->is_parent);
-
-		if ((m->shm != NULL) && (*m->sema == 0) && (m->is_parent == TRUE)) {
-
-			rv = apr_shm_destroy(m->shm);
-			oidc_sdebug(s, "apr_shm_destroy for semaphore returned: %d", rv);
-			m->shm = NULL;
-
-			apr_global_mutex_unlock(m->mutex);
-
-			rv = apr_global_mutex_destroy(m->mutex);
-			oidc_sdebug(s, "apr_global_mutex_destroy returned :%d", rv);
-			m->mutex = NULL;
-
-			rv = APR_SUCCESS;
-
-		} else {
-
-			apr_global_mutex_unlock(m->mutex);
-
-		}
+	if ((m) && (m->is_parent == TRUE) && (m->mutex)) {
+		rv = apr_global_mutex_destroy(m->mutex);
+		oidc_sdebug(s, "apr_global_mutex_destroy returned :%d", rv);
+		m->mutex = NULL;
 	}
 
 	return rv;
