@@ -434,6 +434,42 @@ char* oidc_util_javascript_escape(apr_pool_t *pool, const char *s) {
     return output;
 }
 
+char* oidc_util_strcasestr(const char *s1, const char *s2) {
+	const char *s = s1;
+	const char *p = s2;
+	do {
+		if (!*p)
+			return (char*) s1;
+		if ((*p == *s) || (tolower(*p) == tolower(*s))) {
+			++p;
+			++s;
+		} else {
+			p = s2;
+			if (!*s)
+				return NULL;
+			s = ++s1;
+		}
+	} while (1);
+	return *p ? NULL : (char*) s1;
+}
+
+static const char* oidc_util_hdr_forwarded_get(const request_rec *r, const char *elem) {
+	const char *value = NULL;
+	char *ptr = NULL;
+	const char *item = apr_psprintf(r->pool, "%s=", elem);
+	value = oidc_util_hdr_in_forwarded_get(r);
+	value = oidc_util_strcasestr(value, item);
+	if (value) {
+		value += strlen(item);
+		ptr = strstr(value, ";");
+		if (ptr)
+			*ptr = '\0';
+		ptr = strstr(value, " ");
+		if (ptr)
+			*ptr = '\0';
+	}
+	return apr_pstrdup(r->pool, value);
+}
 
 /*
  * get the URL scheme that is currently being accessed
@@ -443,7 +479,9 @@ static const char* oidc_get_current_url_scheme(const request_rec *r,
 	/* first see if there's a proxy/load-balancer in front of us */
 	const char *scheme_str = NULL;
 
-	if (x_forwarded_headers & OIDC_HDR_X_FORWARDED_PROTO)
+	if ((x_forwarded_headers & OIDC_HDR_FORWARDED))
+		scheme_str = oidc_util_hdr_forwarded_get(r, "proto");
+	else if (x_forwarded_headers & OIDC_HDR_X_FORWARDED_PROTO)
 		scheme_str = oidc_util_hdr_in_x_forwarded_proto_get(r);
 
 	/* if not we'll determine the scheme used to connect to this server */
@@ -505,11 +543,13 @@ static const char* oidc_get_current_url_port(const request_rec *r, const char *s
 		return port_str;
 
 	/*
-	 * see if we can get the port from the "X-Forwarded-Host" header
+	 * see if we can get the port from the "X-Forwarded-Host" or "Forwarded" header
 	 * and if that header was set we'll assume defaults
 	 */
 
-	if (x_forwarded_headers & OIDC_HDR_X_FORWARDED_HOST)
+	if (x_forwarded_headers & OIDC_HDR_FORWARDED)
+		host_hdr = oidc_util_hdr_forwarded_get(r, "host");
+	else if (x_forwarded_headers & OIDC_HDR_X_FORWARDED_HOST)
 		host_hdr = oidc_util_hdr_in_x_forwarded_host_get(r);
 
 	if (host_hdr) {
@@ -540,6 +580,13 @@ static const char* oidc_get_current_url_port(const request_rec *r, const char *s
 		if (oidc_util_hdr_in_x_forwarded_proto_get(r))
 			return NULL;
 
+	/*
+	 * do the same for the Forwarded: proto= header
+	 */
+	if (x_forwarded_headers & OIDC_HDR_FORWARDED)
+		if (oidc_util_hdr_forwarded_get(r, "proto"))
+				return NULL;
+
 	/*
 	 * if no port was set in the Host header and no X-Forwarded-Proto was set, we'll
 	 * determine the port locally and don't print it when it's the default for the protocol
@@ -562,7 +609,9 @@ const char* oidc_get_current_url_host(request_rec *r, const apr_byte_t x_forward
 	char *p = NULL;
 	char *i = NULL;
 
-	if (x_forwarded_headers & OIDC_HDR_X_FORWARDED_HOST)
+	if (x_forwarded_headers & OIDC_HDR_FORWARDED)
+		host_str = oidc_util_hdr_forwarded_get(r, "host");
+	else if (x_forwarded_headers & OIDC_HDR_X_FORWARDED_HOST)
 		host_str = oidc_util_hdr_in_x_forwarded_host_get(r);
 
 	if (host_str == NULL)
@@ -1958,22 +2007,59 @@ apr_byte_t oidc_util_json_array_has_value(request_rec *r, json_t *haystack,
 	return (i == json_array_size(haystack)) ? FALSE : TRUE;
 }
 
+static char* oidc_util_utf8_to_latin1(request_rec *r, const char *src) {
+	char *dst = "";
+	unsigned int cp = 0;
+	unsigned char ch;
+	if (src == NULL)
+		return NULL;
+	while (*src != '\0') {
+		ch = (unsigned char) (*src);
+		if (ch <= 0x7f)
+			cp = ch;
+		else if (ch <= 0xbf)
+			cp = (cp << 6) | (ch & 0x3f);
+		else if (ch <= 0xdf)
+			cp = ch & 0x1f;
+		else if (ch <= 0xef)
+			cp = ch & 0x0f;
+		else
+			cp = ch & 0x07;
+		++src;
+		if (((*src & 0xc0) != 0x80) && (cp <= 0x10ffff)) {
+			if (cp <= 255) {
+				dst = apr_psprintf(r->pool, "%s%c", dst, (unsigned char)cp);
+			} else {
+				// no encoding possible
+				dst = apr_psprintf(r->pool, "%s%c", dst, '?');
+			}
+		}
+	}
+	return dst;
+}
+
+
 /*
  * set a HTTP header and/or environment variable to pass information to the application
  */
 void oidc_util_set_app_info(request_rec *r, const char *s_key,
 		const char *s_value, const char *claim_prefix, apr_byte_t as_header,
-		apr_byte_t as_env_var, apr_byte_t base64url) {
+		apr_byte_t as_env_var, int pass_as) {
 
 	/* construct the header name, cq. put the prefix in front of a normalized key name */
 	const char *s_name = apr_psprintf(r->pool, "%s%s", claim_prefix,
 			oidc_normalize_header_name(r, s_key));
 	char *d_value = NULL;
 
-	if (as_header) {
-		if ((base64url == TRUE) && (s_value != NULL)) {
+	if (s_value != NULL) {
+		if (pass_as == OIDC_PASS_APP_INFO_AS_BASE64URL) {
 			oidc_base64url_encode(r, &d_value, s_value, strlen(s_value), TRUE);
+		} else if (pass_as == OIDC_PASS_APP_INFO_AS_LATIN1) {
+			d_value = oidc_util_utf8_to_latin1(r, s_value);
 		}
+	}
+
+	if (as_header) {
 		oidc_util_hdr_in_set(r, s_name, (d_value != NULL) ? d_value : s_value);
 	}
 
@@ -1981,9 +2067,9 @@ void oidc_util_set_app_info(request_rec *r, const char *s_key,
 
 		/* do some logging about this event */
 		oidc_debug(r, "setting environment variable \"%s: %s\"", s_name,
-				s_value);
+				   (d_value != NULL) ? d_value : s_value);
 
-		apr_table_set(r->subprocess_env, s_name, s_value);
+		apr_table_set(r->subprocess_env, s_name, (d_value != NULL) ? d_value : s_value);
 	}
 }
 
@@ -1992,7 +2078,7 @@ void oidc_util_set_app_info(request_rec *r, const char *s_key,
  */
 void oidc_util_set_app_infos(request_rec *r, const json_t *j_attrs,
 		const char *claim_prefix, const char *claim_delimiter,
-		apr_byte_t as_header, apr_byte_t as_env_var, apr_byte_t base64url) {
+		apr_byte_t as_header, apr_byte_t as_env_var, int pass_as) {
 
 	char s_int[255];
 	json_t *j_value = NULL;
@@ -2021,21 +2107,21 @@ void oidc_util_set_app_infos(request_rec *r, const json_t *j_attrs,
 
 			/* set the single string in the application header whose name is based on the key and the prefix */
 			oidc_util_set_app_info(r, s_key, json_string_value(j_value),
-					claim_prefix, as_header, as_env_var, base64url);
+					claim_prefix, as_header, as_env_var, pass_as);
 
 		} else if (json_is_boolean(j_value)) {
 
 			/* set boolean value in the application header whose name is based on the key and the prefix */
 			oidc_util_set_app_info(r, s_key,
 					(json_is_true(j_value) ? "1" : "0"), claim_prefix,
-					as_header, as_env_var, base64url);
+					as_header, as_env_var, pass_as);
 
 		} else if (json_is_integer(j_value)) {
 
 			if (sprintf(s_int, "%ld", (long) json_integer_value(j_value)) > 0) {
 				/* set long value in the application header whose name is based on the key and the prefix */
 				oidc_util_set_app_info(r, s_key, s_int, claim_prefix, as_header,
-						as_env_var, base64url);
+						as_env_var, pass_as);
 			} else {
 				oidc_warn(r,
 						"could not convert JSON number to string (> 255 characters?), skipping");
@@ -2046,14 +2132,14 @@ void oidc_util_set_app_infos(request_rec *r, const json_t *j_attrs,
 			/* set float value in the application header whose name is based on the key and the prefix */
 			oidc_util_set_app_info(r, s_key,
 					apr_psprintf(r->pool, "%lf", json_real_value(j_value)),
-					claim_prefix, as_header, as_env_var, base64url);
+					claim_prefix, as_header, as_env_var, pass_as);
 
 		} else if (json_is_object(j_value)) {
 
 			/* set json value in the application header whose name is based on the key and the prefix */
 			oidc_util_set_app_info(r, s_key,
 					oidc_util_encode_json_object(r, j_value, 0), claim_prefix,
-					as_header, as_env_var, base64url);
+					as_header, as_env_var, pass_as);
 
 			/* check if it is a multi-value string */
 		} else if (json_is_array(j_value)) {
@@ -2108,7 +2194,7 @@ void oidc_util_set_app_infos(request_rec *r, const json_t *j_attrs,
 
 			/* set the concatenated string */
 			oidc_util_set_app_info(r, s_key, s_concat, claim_prefix, as_header,
-					as_env_var, base64url);
+					as_env_var, pass_as);
 
 		} else {
 
@@ -2629,6 +2715,11 @@ const char* oidc_util_hdr_in_x_forwarded_host_get(const request_rec *r) {
 			OIDC_HTTP_HDR_X_FORWARDED_HOST, OIDC_STR_COMMA OIDC_STR_SPACE);
 }
 
+const char* oidc_util_hdr_in_forwarded_get(const request_rec *r) {
+	return oidc_util_hdr_in_get_left_most_only(r,
+			OIDC_HTTP_HDR_FORWARDED, OIDC_STR_COMMA);
+}
+
 const char* oidc_util_hdr_in_host_get(const request_rec *r) {
 	return oidc_util_hdr_in_get(r, OIDC_HTTP_HDR_HOST);
 }
